"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Attribute = void 0;
const _ = __importStar(require("lodash"));
const errors_1 = require("./errors");
const truly_empty_1 = require("./utils/truly-empty");
class Attribute {
    constructor(propertyName, type, metadata = {}) {
        this.propertyName = propertyName;
        this.type = type;
        this.metadata = metadata;
        this.name = this.metadata.name == null ? this.propertyName : this.metadata.name;
    }
    /**
     * Set the default value for an attribute, if no value is currently set
     */
    getDefaultValue() {
        if (typeof this.metadata.default !== 'undefined') {
            return _.isFunction(this.metadata.default) ? this.metadata.default() : this.metadata.default;
        }
        else if (typeof this.type.getDefault === 'function') {
            return this.type.getDefault();
        }
        else {
            return null;
        }
    }
    /**
     * Convert the given value for this attribute to a DynamoDB AttributeValue
     */
    toDynamo(value) {
        // if there is no value, inject the default value for this attribute
        if (value == null || (0, truly_empty_1.isTrulyEmpty)(value)) {
            // if we have no value, allow the manipulateWrite a chance to provide a value
            if (typeof this.metadata.manipulateWrite === 'function') {
                return this.metadata.manipulateWrite(null, null, this);
            }
            else {
                return null;
            }
        }
        // if there is no value, do not not return an empty AttributeValue
        if (value == null) {
            if (this.metadata.required === true) {
                throw new errors_1.ValidationError('Required value missing: ' + this.name);
            }
            return null;
        }
        if (typeof this.metadata.validate === 'function' && !this.metadata.validate(value)) {
            throw new errors_1.ValidationError('Validation failed: ' + this.name);
        }
        const attributeValue = this.type.toDynamo(value, this);
        if (typeof this.metadata.manipulateWrite === 'function') {
            return this.metadata.manipulateWrite(attributeValue, value, this);
        }
        else {
            return attributeValue;
        }
    }
    toDynamoAssert(value) {
        const attributeValue = this.toDynamo(value);
        if (attributeValue == null) {
            throw new errors_1.ValidationError(`Attribute.toDynamoAssert called without a valid value for ${this.name}`);
        }
        else {
            return attributeValue;
        }
    }
    /**
     * Convert DynamoDB raw response to understandable data
     */
    fromDynamo(attributeValue) {
        // if there is no value, apply the default, but allow the value to become null
        if (attributeValue == null) {
            attributeValue = this.toDynamo(null);
        }
        // all attributes support null
        if (attributeValue == null || attributeValue.NULL === true) {
            if (typeof this.metadata.manipulateRead === 'function') {
                return this.metadata.manipulateRead(null, attributeValue, this);
            }
            else {
                return null;
            }
        }
        const value = this.type.fromDynamo(attributeValue, this);
        if (typeof this.metadata.manipulateRead === 'function') {
            return this.metadata.manipulateRead(value, attributeValue, this);
        }
        else {
            return value;
        }
    }
}
exports.Attribute = Attribute;
//# sourceMappingURL=attribute.js.map