"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinarySetAttributeType = void 0;
const lodash_1 = require("lodash");
const errors_1 = require("../../errors");
const attribute_type_1 = require("../../tables/attribute-type");
class BinarySetAttributeType extends attribute_type_1.AttributeType {
    constructor() {
        super(...arguments);
        this.type = "BS" /* DynamoAttributeType.BinarySet */;
    }
    toDynamo(values) {
        if (!(0, lodash_1.isSet)(values) && !(0, lodash_1.isArray)(values)) {
            throw new errors_1.ValidationError(`Expected ${this.propertyName} to be an array of binary values`);
        }
        return {
            BS: Array.from(values),
        };
    }
    fromDynamo(value) {
        var _a;
        // this needs to return null when there is no value, so the default value can be set if necessary
        // returning an empty array means there was a value from DynamoDB with a Set containing no items
        if (value.BS == null) {
            return null;
        }
        else if (((_a = this.metadata) === null || _a === void 0 ? void 0 : _a.array) === true) {
            return value.BS;
        }
        else {
            const binarySet = new Set();
            value.BS.forEach((item) => {
                binarySet.add(item);
            });
            return binarySet;
        }
    }
}
exports.BinarySetAttributeType = BinarySetAttributeType;
//# sourceMappingURL=binary-set.js.map