"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NumberSetAttributeType = void 0;
const lodash_1 = require("lodash");
const errors_1 = require("../../errors");
const attribute_type_1 = require("../../tables/attribute-type");
const utils_1 = require("./utils");
class NumberSetAttributeType extends attribute_type_1.AttributeType {
    constructor() {
        super(...arguments);
        this.type = "NS" /* DynamoAttributeType.NumberSet */;
    }
    toDynamo(values) {
        if ((0, utils_1.isNumber)(values)) {
            return { NS: [(0, utils_1.numberToString)(values)] };
        }
        if ((!(0, lodash_1.isSet)(values) && !(0, lodash_1.isArray)(values)) || !(0, lodash_1.every)(values, utils_1.isNumber)) {
            throw new errors_1.ValidationError(`Expected ${this.propertyName} to be an array of number values`);
        }
        return {
            NS: (0, lodash_1.uniq)(Array.from(values).map((value) => (0, utils_1.numberToString)(value))),
        };
    }
    fromDynamo(value) {
        // this needs to return null when there is no value, so the default value can be set if necessary
        // returning an empty array means there was a value from DynamoDB with a Set containing no items
        if (value.NS == null) {
            return null;
        }
        else {
            return this.fromJSON(value.NS);
        }
    }
    toJSON(values) {
        return Array.from(values);
    }
    fromJSON(values) {
        var _a;
        if (((_a = this.metadata) === null || _a === void 0 ? void 0 : _a.array) === true) {
            return values.map((item) => (0, utils_1.stringToNumber)(item));
        }
        else {
            const numberSet = new Set();
            values.forEach((item) => {
                numberSet.add((0, utils_1.stringToNumber)(item));
            });
            return numberSet;
        }
    }
}
exports.NumberSetAttributeType = NumberSetAttributeType;
//# sourceMappingURL=number-set.js.map