"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringSetAttributeType = void 0;
const lodash_1 = require("lodash");
const errors_1 = require("../../errors");
const attribute_type_1 = require("../../tables/attribute-type");
class StringSetAttributeType extends attribute_type_1.AttributeType {
    constructor() {
        super(...arguments);
        this.type = "SS" /* DynamoAttributeType.StringSet */;
    }
    toDynamo(values) {
        if ((0, lodash_1.isString)(values)) {
            return { SS: [values] };
        }
        if ((!(0, lodash_1.isSet)(values) && !(0, lodash_1.isArray)(values)) || !(0, lodash_1.every)(values, lodash_1.isString)) {
            throw new errors_1.ValidationError(`Expected ${this.propertyName} to be a set of string values`);
        }
        return {
            SS: (0, lodash_1.uniq)(Array.from(values)),
        };
    }
    fromDynamo(value) {
        var _a;
        // this needs to return null when there is no value, so the default value can be set if necessary
        // returning an empty array means there was a value from DynamoDB with a Set containing no items
        if (value.SS == null) {
            return null;
        }
        else if (((_a = this.metadata) === null || _a === void 0 ? void 0 : _a.array) === true) {
            return value.SS;
        }
        else {
            const stringSet = new Set();
            value.SS.forEach((item) => {
                stringSet.add(item);
            });
            return stringSet;
        }
    }
    toJSON(value) {
        if ((0, lodash_1.isSet)(value)) {
            return Array.from(value);
        }
        else {
            return value;
        }
    }
}
exports.StringSetAttributeType = StringSetAttributeType;
//# sourceMappingURL=string-set.js.map