"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUpdateItemInput = void 0;
const _ = __importStar(require("lodash"));
const expression_1 = require("./expression");
function getUpdateItemInput(record, params) {
    var _a;
    const tableClass = record.constructor;
    const input = {
        TableName: tableClass.schema.name,
        Key: record.getDynamoKey(),
        ReturnValues: (_a = params === null || params === void 0 ? void 0 : params.returnValues) !== null && _a !== void 0 ? _a : 'NONE',
    };
    if ((params === null || params === void 0 ? void 0 : params.returnConsumedCapacity) != null) {
        input.ReturnConsumedCapacity = params.returnConsumedCapacity;
    }
    const sets = [];
    const adds = [];
    const deletes = [];
    const removes = [];
    const attributeNameMap = {};
    const attributeValueMap = {};
    let valueCounter = 0;
    // we call toDynamo to have the record self-check for any dynamic attributes
    record.toDynamo(true);
    _.each(_.uniq(record.getUpdatedAttributes()), (attributeName, i) => {
        const attribute = tableClass.schema.getAttributeByName(attributeName);
        const value = attribute.toDynamo(record.getAttribute(attributeName));
        const operator = record.getAttributeUpdateOperator(attributeName);
        const slug = `#UA${valueCounter}`;
        if (value != null) {
            attributeNameMap[slug] = attributeName;
            attributeValueMap[`:u${valueCounter}`] = value;
            switch (operator) {
                // Number attribute operators
                case 'increment':
                    sets.push(`${slug} = ${slug} + :u${valueCounter}`);
                    break;
                case 'decrement':
                    sets.push(`${slug} = ${slug} - :u${valueCounter}`);
                    break;
                // List attribute operators
                case 'append':
                    sets.push(`${slug} = list_append(${slug}, :u${valueCounter})`);
                    break;
                case 'if_not_exists':
                    sets.push(`${slug} = if_not_exists(${slug}, :u${valueCounter})`);
                    break;
                // Set attribute operators
                case 'add':
                    adds.push(`${slug} :u${valueCounter}`);
                    break;
                case 'delete':
                    deletes.push(`${slug} :u${valueCounter}`);
                    break;
                case 'set':
                default:
                    sets.push(`${slug} = :u${valueCounter}`);
                    break;
            }
            valueCounter++;
        }
    });
    _.each(_.uniq(record.getRemovedAttributes()), (attrName, i) => {
        const slug = `#DA${valueCounter}`;
        attributeNameMap[slug] = attrName;
        removes.push(slug);
        valueCounter++;
    });
    let updateExpression = '';
    if (sets.length > 0) {
        updateExpression += 'SET ' + sets.join(', ');
    }
    if (adds.length > 0) {
        if (updateExpression.length > 0) {
            updateExpression += ' ';
        }
        updateExpression += 'ADD ' + adds.join(', ');
    }
    if (deletes.length > 0) {
        if (updateExpression.length > 0) {
            updateExpression += ' ';
        }
        updateExpression += 'DELETE ' + deletes.join(', ');
    }
    if (removes.length > 0) {
        if (updateExpression.length > 0) {
            updateExpression += ' ';
        }
        updateExpression += 'REMOVE ' + removes.join(', ');
    }
    if ((params === null || params === void 0 ? void 0 : params.conditions) != null) {
        const conditionExpression = (0, expression_1.buildQueryExpression)(tableClass.schema, params.conditions);
        input.ConditionExpression = conditionExpression.FilterExpression;
        Object.assign(attributeNameMap, conditionExpression.ExpressionAttributeNames);
        Object.assign(attributeValueMap, conditionExpression.ExpressionAttributeValues);
    }
    input.ExpressionAttributeNames = attributeNameMap;
    input.UpdateExpression = updateExpression;
    if (_.size(attributeValueMap) > 0) {
        input.ExpressionAttributeValues = attributeValueMap;
    }
    return input;
}
exports.getUpdateItemInput = getUpdateItemInput;
//# sourceMappingURL=update-item-input.js.map