"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTable = void 0;
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
async function createTable(schema, waitForReady = true) {
    const res = await schema.dynamo.createTable(schema.createTableInput());
    if (waitForReady) {
        await (0, client_dynamodb_1.waitUntilTableExists)({ client: schema.dynamo, maxWaitTime: 500 }, { TableName: schema.name });
        // TTL
        if (schema.timeToLiveAttribute != null) {
            await schema.dynamo.updateTimeToLive({
                TableName: schema.name,
                TimeToLiveSpecification: {
                    Enabled: true,
                    AttributeName: schema.timeToLiveAttribute.name,
                },
            });
        }
        // Point-in-Time Recovery
        if (schema.options.backup === true) {
            await schema.dynamo.updateContinuousBackups({
                TableName: schema.name,
                PointInTimeRecoverySpecification: {
                    PointInTimeRecoveryEnabled: true,
                },
            });
        }
    }
    return res.TableDescription;
}
exports.createTable = createTable;
//# sourceMappingURL=create-table.js.map