"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.migrateTable = void 0;
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const _ = __importStar(require("lodash"));
const errors_1 = require("../errors");
const create_table_1 = require("./create-table");
const describe_table_1 = require("./describe-table");
async function migrateTable(schema, waitForReady = false) {
    let description;
    try {
        description = await (0, describe_table_1.describeTable)(schema);
    }
    catch (err) {
        if (err.name === 'ResourceNotFoundException') {
            return await (0, create_table_1.createTable)(schema, waitForReady);
        }
        else {
            throw err;
        }
    }
    const expectedDescription = schema.createTableInput();
    const indexes = [];
    const expectedIndexes = [];
    if (description.GlobalSecondaryIndexes != null) {
        description.GlobalSecondaryIndexes.forEach((index) => {
            if (index.IndexName != null) {
                indexes.push(index.IndexName);
            }
        });
    }
    if (expectedDescription.GlobalSecondaryIndexes != null) {
        expectedDescription.GlobalSecondaryIndexes.forEach((index) => {
            if (index.IndexName != null) {
                expectedIndexes.push(index.IndexName);
            }
        });
    }
    const deletedIndexes = [];
    let hasChanges = indexes.length < expectedIndexes.length;
    if (!hasChanges) {
        _.each(description.GlobalSecondaryIndexes, (oldIndex) => {
            const newIndex = _.find(expectedDescription.GlobalSecondaryIndexes, i => i.IndexName === oldIndex.IndexName);
            if (newIndex == null) {
                deletedIndexes.push(oldIndex);
                hasChanges = true;
            }
            else if (!_.isEqual(oldIndex.KeySchema, newIndex.KeySchema) ||
                // assign a default value for NonKeyAttributes to prevent comparison issues, fixes #645
                !_.isEqual({ NonKeyAttributes: undefined, ...oldIndex.Projection }, { NonKeyAttributes: undefined, ...newIndex.Projection })) {
                const oldIndexName = oldIndex.IndexName == null ? '' : oldIndex.IndexName;
                // you can only updated ProvisionedThroughput, which is useless to do on the DynamoDB development server
                // so really we want to verify we're not attempting to change an index's KeySchema or Projection, if we
                // are, error and warn developer… they need to rename the index so the old one is deleted
                throw new errors_1.SchemaError(`Cannot update KeySchema or Projection for ${oldIndexName}, you must rename the index to delete the one and create a new one`);
            }
        });
    }
    if (hasChanges) {
        const newIndexes = _.filter(expectedDescription.GlobalSecondaryIndexes, (index) => {
            return !_.includes(indexes, index.IndexName);
        });
        const indexUpdates = [];
        _.forEach(deletedIndexes, (index) => {
            if (index.IndexName != null) {
                indexUpdates.push({ Delete: { IndexName: index.IndexName } });
            }
        });
        _.forEach(newIndexes, (index) => {
            const action = index;
            indexUpdates.push({ Create: action });
        });
        // you can only update or delete a single index per request, even though the command accepts an array
        for (const indexUpdate of indexUpdates) {
            const updateParams = {
                TableName: expectedDescription.TableName,
                AttributeDefinitions: expectedDescription.AttributeDefinitions,
                GlobalSecondaryIndexUpdates: [indexUpdate],
            };
            // tell the table to update this index
            await schema.dynamo.updateTable(updateParams);
            // now wait for the index to be ready
            await (0, client_dynamodb_1.waitUntilTableExists)({ client: schema.dynamo, maxWaitTime: 500 }, { TableName: schema.name });
        }
        // TTL
        const ttl = await schema.dynamo.describeTimeToLive({ TableName: schema.name });
        if (
        // if there currently is no TTL attribute but we want one
        (ttl.TimeToLiveDescription == null && schema.timeToLiveAttribute != null) ||
            // or if the TTL attribute has changed
            (ttl.TimeToLiveDescription != null && schema.timeToLiveAttribute != null && ttl.TimeToLiveDescription.AttributeName !== schema.timeToLiveAttribute.name)
        // TODO: if there is a TTL attribute but we no longer want one, we should delete it
        ) {
            await schema.dynamo.updateTimeToLive({
                TableName: schema.name,
                TimeToLiveSpecification: {
                    Enabled: true,
                    AttributeName: schema.timeToLiveAttribute.name,
                },
            });
        }
    }
    return description;
}
exports.migrateTable = migrateTable;
//# sourceMappingURL=migrate-table.js.map