"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Schema = void 0;
const errors_1 = require("../errors");
const Query = __importStar(require("../query"));
const create_table_input_1 = require("./create-table-input");
class Schema {
    /**
     * The TableName in DynamoDB
     */
    get name() {
        var _a;
        return ((_a = this.options) === null || _a === void 0 ? void 0 : _a.name) == null ? '' : this.options.name;
    }
    constructor(table) {
        this.table = table;
        this.isDyngoose = true;
        // Additional table indexes
        this.globalSecondaryIndexes = [];
        this.localSecondaryIndexes = [];
        // List of attributes this table has
        this.attributes = new Map();
    }
    setMetadata(metadata) {
        this.options = Object.assign({
            // default options for a table
            billingMode: 'PAY_PER_REQUEST',
            backup: true,
        }, metadata);
        if (this.options.throughput != null) {
            this.setThroughput(this.options.throughput);
        }
    }
    defineAttributeProperties() {
        // for each attribute, add the get and set property handlers
        for (const attribute of this.attributes.values()) {
            if (Object.prototype.hasOwnProperty.call(this.table, attribute.propertyName) === false ||
                // every function in JavaScript has a 'name' property, however, name is commonly
                // used as an attribute name so we basically want to ignore the default object property
                // … I know, a weird exception
                attribute.propertyName === 'name') {
                Object.defineProperty(this.table.prototype, attribute.propertyName, {
                    configurable: true,
                    enumerable: true,
                    get() {
                        return this.getAttribute(attribute.name);
                    },
                    set(value) {
                        this.setAttribute(attribute.name, value);
                    },
                });
            }
        }
    }
    defineGlobalSecondaryIndexes() {
        for (const indexMetadata of this.globalSecondaryIndexes) {
            if (Object.prototype.hasOwnProperty.call(this.table, indexMetadata.propertyName) === false) {
                Object.defineProperty(this.table, indexMetadata.propertyName, {
                    value: new Query.GlobalSecondaryIndex(this.table, indexMetadata),
                    writable: false,
                });
            }
        }
    }
    defineLocalSecondaryIndexes() {
        for (const indexMetadata of this.localSecondaryIndexes) {
            if (Object.prototype.hasOwnProperty.call(this.table, indexMetadata.propertyName) === false) {
                Object.defineProperty(this.table, indexMetadata.propertyName, {
                    value: new Query.LocalSecondaryIndex(this.table, indexMetadata),
                    writable: false,
                });
            }
        }
    }
    definePrimaryKeyProperty() {
        if (Object.prototype.hasOwnProperty.call(this.table, this.primaryKey.propertyName) === false) {
            Object.defineProperty(this.table, this.primaryKey.propertyName, {
                value: new Query.PrimaryKey(this.table, this.primaryKey),
                writable: false,
            });
        }
    }
    setThroughput(throughput) {
        if (typeof throughput === 'number') {
            this.throughput = {
                read: throughput,
                write: throughput,
            };
        }
        else {
            this.throughput = throughput;
        }
        if (this.throughput.read == null || this.throughput.write == null) {
            throw new errors_1.SchemaError(`Schema for ${this.name} has invalid throughput ${JSON.stringify(this.throughput)}`);
        }
    }
    getAttributes() {
        return this.attributes.entries();
    }
    getAttributeByName(attributeName) {
        let attribute;
        if (attributeName.includes('.')) {
            const nameSegments = attributeName.split('.');
            const firstSegment = nameSegments.shift();
            if (firstSegment != null) {
                attribute = this.attributes.get(firstSegment);
                for (const nameSegment of nameSegments) {
                    if (attribute != null) {
                        attribute = attribute.type.attributes[nameSegment];
                    }
                }
            }
        }
        else {
            attribute = this.attributes.get(attributeName);
        }
        if (attribute == null) {
            throw new errors_1.SchemaError(`Schema for ${this.name} has no attribute named ${attributeName}`);
        }
        else {
            return attribute;
        }
    }
    getAttributeByPropertyName(propertyName) {
        let attribute;
        if (propertyName.includes('.')) {
            const nameSegments = propertyName.split('.');
            const firstSegment = nameSegments.shift();
            if (firstSegment != null) {
                for (const attr of this.attributes.values()) {
                    if (attr.propertyName === firstSegment) {
                        attribute = attr;
                    }
                }
                for (const nameSegment of nameSegments) {
                    if (attribute != null) {
                        const mapAttributes = attribute.type.attributes;
                        mapAttributesFor: for (const mapAttribute of Object.values(mapAttributes)) {
                            if (mapAttribute.propertyName === nameSegment) {
                                attribute = mapAttribute;
                                break mapAttributesFor;
                            }
                        }
                    }
                }
            }
        }
        else {
            for (const attr of this.attributes.values()) {
                if (attr.propertyName === propertyName) {
                    attribute = attr;
                }
            }
        }
        if (attribute == null) {
            throw new errors_1.SchemaError(`Schema for ${this.name} has no attribute by property name ${propertyName}`);
        }
        else {
            return attribute;
        }
    }
    addAttribute(attribute) {
        if (this.attributes.has(attribute.name)) {
            throw new errors_1.SchemaError(`Table ${this.name} has several attributes named ${attribute.name}`);
        }
        this.attributes.set(attribute.name, attribute);
        return attribute;
    }
    setPrimaryKey(primaryKey, sortKey, propertyName) {
        const hash = this.getAttributeByName(primaryKey);
        if (hash == null) {
            throw new errors_1.SchemaError(`Specified primaryKey ${primaryKey} attribute for the PrimaryKey for table ${this.name} does not exist`);
        }
        let range;
        if (sortKey != null) {
            range = this.getAttributeByName(sortKey);
            if (range == null) {
                throw new errors_1.SchemaError(`Specified sortKey ${sortKey} attribute for the PrimaryKey for table ${this.name} does not exist`);
            }
        }
        this.primaryKey = {
            propertyName,
            hash,
            range,
        };
    }
    createTableInput(forCloudFormation = false) {
        return (0, create_table_input_1.createTableInput)(this, forCloudFormation);
    }
    createCloudFormationResource() {
        return this.createTableInput(true);
    }
    toDynamo(record) {
        const attributeMap = {};
        for (const [attributeName, attribute] of this.attributes.entries()) {
            // there is a quirk with the typing of Table.get, where we exclude all the default Table properties and therefore
            // on the Table class itself, no property name is possible, so we pass 'as never' below to fix a linter warning
            // but this actually works as expected
            const attributeValue = attribute.toDynamo(record.get(attribute.propertyName));
            if (attributeValue != null) {
                attributeMap[attributeName] = attributeValue;
            }
        }
        return attributeMap;
    }
}
exports.Schema = Schema;
//# sourceMappingURL=schema.js.map