"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCDKTables = exports.createCDKTable = void 0;
const dynamodb = __importStar(require("aws-cdk-lib/aws-dynamodb"));
const promises_1 = require("fs/promises");
const lodash_1 = require("lodash");
const path_1 = require("path");
const defaults_1 = require("../tables/defaults");
const table_1 = require("../table");
function createCDKTable(scope, schema, tableProps = {}) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
    if (tableProps.tableName == null) {
        tableProps.tableName = schema.name;
    }
    if (tableProps.partitionKey == null) {
        tableProps.partitionKey = {
            name: schema.primaryKey.hash.name,
            type: schema.primaryKey.hash.type.type,
        };
    }
    if (tableProps.sortKey == null && schema.primaryKey.range != null) {
        tableProps.sortKey = {
            name: schema.primaryKey.range.name,
            type: schema.primaryKey.range.type.type,
        };
    }
    if (tableProps.billingMode == null) {
        if (schema.options.billingMode === 'PAY_PER_REQUEST') {
            tableProps.billingMode = dynamodb.BillingMode.PAY_PER_REQUEST;
        }
        else if (schema.options.billingMode === 'PROVISIONED') {
            tableProps.billingMode = dynamodb.BillingMode.PROVISIONED;
            tableProps.readCapacity = (_b = (_a = schema.throughput) === null || _a === void 0 ? void 0 : _a.read) !== null && _b !== void 0 ? _b : defaults_1.DEFAULT_READ_CAPACITY;
            tableProps.writeCapacity = (_d = (_c = schema.throughput) === null || _c === void 0 ? void 0 : _c.write) !== null && _d !== void 0 ? _d : defaults_1.DEFAULT_WRITE_CAPACITY;
        }
    }
    if (tableProps.pointInTimeRecovery == null && schema.options.backup === true) {
        tableProps.pointInTimeRecovery = true;
    }
    if (tableProps.encryption == null && schema.options.encrypted === true) {
        tableProps.encryption = dynamodb.TableEncryption.AWS_MANAGED;
    }
    if (tableProps.timeToLiveAttribute == null && schema.timeToLiveAttribute != null) {
        tableProps.timeToLiveAttribute = schema.timeToLiveAttribute.name;
    }
    if (tableProps.stream == null && schema.options.stream != null) {
        if (typeof schema.options.stream === 'boolean') {
            tableProps.stream = dynamodb.StreamViewType.NEW_AND_OLD_IMAGES;
        }
        else if (schema.options.stream.StreamViewType != null) {
            tableProps.stream = schema.options.stream.StreamViewType;
        }
    }
    const table = new dynamodb.Table(scope, `${schema.name}Table`, tableProps);
    if (schema.options.billingMode === 'PROVISIONED') {
        let autoScaling = (_e = schema.throughput) === null || _e === void 0 ? void 0 : _e.autoScaling;
        if (autoScaling === true) {
            // default values will be applied automatically below
            autoScaling = {};
        }
        if (autoScaling != null) {
            const autoScalingAny = autoScaling;
            const read = (0, lodash_1.isNil)(autoScalingAny.read) ? autoScalingAny : autoScalingAny.read;
            const write = (0, lodash_1.isNil)(autoScalingAny.write) ? autoScalingAny : autoScalingAny.write;
            table.autoScaleReadCapacity({
                minCapacity: (_f = read.minCapacity) !== null && _f !== void 0 ? _f : defaults_1.DEFAULT_AUTOSCALING_MIN_CAPACITY,
                maxCapacity: (_g = read.maxCapacity) !== null && _g !== void 0 ? _g : defaults_1.DEFAULT_AUTOSCALING_MAX_CAPACITY,
            }).scaleOnUtilization({
                targetUtilizationPercent: (_h = read.targetUtilization) !== null && _h !== void 0 ? _h : defaults_1.DEFAULT_AUTOSCALING_TARGET_UTILIZATION,
            });
            table.autoScaleWriteCapacity({
                minCapacity: (_j = write.minCapacity) !== null && _j !== void 0 ? _j : defaults_1.DEFAULT_AUTOSCALING_MIN_CAPACITY,
                maxCapacity: (_k = write.maxCapacity) !== null && _k !== void 0 ? _k : defaults_1.DEFAULT_AUTOSCALING_MIN_CAPACITY,
            }).scaleOnUtilization({
                targetUtilizationPercent: (_l = write.targetUtilization) !== null && _l !== void 0 ? _l : defaults_1.DEFAULT_AUTOSCALING_TARGET_UTILIZATION,
            });
        }
    }
    if (schema.localSecondaryIndexes.length > 0) {
        for (const indexMetadata of schema.localSecondaryIndexes) {
            table.addLocalSecondaryIndex({
                indexName: indexMetadata.name,
                projectionType: indexMetadata.projection,
                sortKey: {
                    name: indexMetadata.range.name,
                    type: indexMetadata.range.type.type,
                },
                nonKeyAttributes: indexMetadata.nonKeyAttributes,
            });
        }
    }
    if (schema.globalSecondaryIndexes.length > 0) {
        for (const indexMetadata of schema.globalSecondaryIndexes) {
            table.addGlobalSecondaryIndex({
                indexName: indexMetadata.name,
                partitionKey: {
                    name: indexMetadata.hash.name,
                    type: indexMetadata.hash.type.type,
                },
                sortKey: indexMetadata.range == null
                    ? undefined
                    : { name: indexMetadata.range.name, type: indexMetadata.range.type.type },
                nonKeyAttributes: indexMetadata.nonKeyAttributes,
                projectionType: indexMetadata.projection,
                readCapacity: indexMetadata.throughput == null ? tableProps.readCapacity : indexMetadata.throughput.read,
                writeCapacity: indexMetadata.throughput == null ? tableProps.writeCapacity : indexMetadata.throughput.write,
            });
        }
    }
    return table;
}
exports.createCDKTable = createCDKTable;
/**
 * Convert a folder of tables into an array of CDK Table constructs.
 */
async function createCDKTables(input) {
    const tableFiles = await (0, promises_1.readdir)(input.tablesDirectory);
    const tables = [];
    const resources = [];
    const log = input.log == null ? console.log : input.log;
    log('Running Dyngoose CDK generation utility…');
    for (const file of tableFiles) {
        if (file.endsWith(input.tableFileSuffix)) {
            const tableFile = (0, path_1.join)(input.tablesDirectory, file);
            const tableFileExports = require(tableFile);
            for (const exportedProperty of Object.values(tableFileExports)) {
                if (exportedProperty.prototype instanceof table_1.Table) {
                    tables.push(exportedProperty);
                }
            }
        }
    }
    for (const SomeTable of tables) {
        const table = createCDKTable(input.scope, SomeTable.schema);
        log(`Generated ${table.tableName} CDK Table`);
        resources.push(table);
    }
    return resources;
}
exports.createCDKTables = createCDKTables;
//# sourceMappingURL=cdk.js.map